# -*- coding: utf-8 -*-
from dataclasses import dataclass
from typing import List, Optional

from themoviedb.schemas._enums import SizeType
from themoviedb.schemas._partial import PartialEpisode
from themoviedb.schemas.credits import Cast, Crew


@dataclass
class Episode(PartialEpisode):
    crew: Optional[List[Crew]] = None
    guest_stars: Optional[List[Cast]] = None

    def __str__(self) -> str:
        return self.name or ""

    def still_url(self, size: SizeType = SizeType.original) -> Optional[str]:
        return f"https://image.tmdb.org/t/p/{size.value}{self.still_path}" if self.still_path else None



# -*- coding: utf-8 -*-
from dataclasses import dataclass
from datetime import datetime
from typing import List, Optional

from themoviedb.schemas._result import ResultWithID


@dataclass
class ReleaseDateLocalized:
    certification: Optional[str] = None
    iso_639_1: Optional[str] = None
    release_date: Optional[datetime] = None
    type: Optional[int] = None
    descriptors: Optional[List[str]] = None
    note: Optional[str] = None


@dataclass
class ReleaseDate:
    iso_3166_1: Optional[str] = None
    release_dates: Optional[List[ReleaseDateLocalized]] = None


@dataclass
class ReleaseDates(ResultWithID):
    results: Optional[List[ReleaseDate]] = None



# -*- coding: utf-8 -*-
from typing import Optional

from themoviedb import schemas, utils
from themoviedb.routes_async._base import Base


class Person(Base):
    def __init__(self, person_id: int, **kwargs) -> None:
        super().__init__(**kwargs)
        self.person_id = person_id

    async def details(
        self, *, append_to_response: Optional[str] = None, image_language: str = "null"
    ) -> schemas.Person:
        """Get the primary person details by id.

        See more: https://developers.themoviedb.org/3/people/get-person-details
        """
        data = await self.request(
            f"person/{self.person_id}", append_to_response=append_to_response, include_image_language=image_language
        )
        return utils.as_dataclass(schemas.Person, data)

    async def external_ids(self) -> schemas.ExternalIDs:
        """Get the external ids for a person.

        See more: https://developers.themoviedb.org/3/people/get-person-external-ids
        """
        data = await self.request(f"person/{self.person_id}/external_ids")
        return utils.as_dataclass(schemas.ExternalIDs, data)

    async def images(self) -> schemas.Images:
        """Get the images for a person.

        See more: https://developers.themoviedb.org/3/people/get-person-images
        """
        data = await self.request(f"person/{self.person_id}/images")
        return utils.as_dataclass(schemas.Images, data)

    async def combined_credits(self) -> schemas.CreditsCombined:
        """Get the movie and TV credits together in a single response.

        See more: https://developers.themoviedb.org/3/people/get-person-combined-credits
        """
        data = await self.request(f"person/{self.person_id}/combined_credits")
        return utils.as_dataclass(schemas.CreditsCombined, data)

    async def movie_credits(self) -> schemas.CreditsMovie:
        """Get the movie credits for a person.

        See more: https://developers.themoviedb.org/3/people/get-person-movie-credits
        """
        data = await self.request(f"person/{self.person_id}/movie_credits")
        return utils.as_dataclass(schemas.CreditsMovie, data)

    async def tv_credits(self) -> schemas.CreditsTV:
        """Get the TV show credits for a person.

        See more: https://developers.themoviedb.org/3/people/get-person-tv-credits
        """
        data = await self.request(f"person/{self.person_id}/tv_credits")
        return utils.as_dataclass(schemas.CreditsTV, data)

    async def tagged_images(self) -> schemas.TaggedImages:
        """Get the images that this person has been tagged in.

        See more: https://developers.themoviedb.org/3/people/get-tagged-images
        """
        data = await self.request(f"person/{self.person_id}/tagged_images")
        return utils.as_dataclass(schemas.TaggedImages, data)

    async def translations(self) -> schemas.Translations:
        """Get a list of translations that have been created for a person.

        See more: https://developers.themoviedb.org/3/people/get-person-translations
        """
        data = await self.request(f"person/{self.person_id}/translations")
        return utils.as_dataclass(schemas.Translations, data)


class People(Base):
    async def latest(self) -> schemas.Person:
        """Get the most newly created person.

        See more: https://developers.themoviedb.org/3/people/get-latest-person
        """
        data = await self.request("person/latest")
        return utils.as_dataclass(schemas.Person, data)

    async def popular(self, *, page: int = 1) -> schemas.People:
        """Get the list of popular people on TMDB.

        See more: https://developers.themoviedb.org/3/people/get-popular-people
        """
        data = await self.request("person/popular", page=page)
        return utils.as_dataclass(schemas.People, data)



# -*- coding: utf-8 -*-
from themoviedb import schemas, utils
from themoviedb.routes_async._base import Base


class Genres(Base):
    async def movie(self) -> schemas.Genres:
        """Get the list of official genres for movies.

        See more: https://developers.themoviedb.org/3/genres/get-movie-list
        """
        data = await self.request("genre/movie/list")
        return utils.as_dataclass(schemas.Genres, data)

    async def tv(self) -> schemas.Genres:
        """Get the list of official genres for TV shows.

        See more: https://developers.themoviedb.org/3/genres/get-tv-list
        """
        data = await self.request("genre/tv/list")
        return utils.as_dataclass(schemas.Genres, data)



# -*- coding: utf-8 -*-
from typing import Optional

from themoviedb import schemas, utils
from themoviedb.routes_sync._base import Base


class Movie(Base):
    def __init__(self, movie_id: int, **kwargs) -> None:
        super().__init__(**kwargs)
        self.movie_id = movie_id

    def details(self, *, append_to_response: Optional[str] = None, image_language: str = "null") -> schemas.Movie:
        """Get the primary information about a movie.

        See more: https://developers.themoviedb.org/3/movies/get-movie-details
        """
        data = self.request(
            f"movie/{self.movie_id}", append_to_response=append_to_response, include_image_language=image_language
        )
        return utils.as_dataclass(schemas.Movie, data)

    def alternative_titles(self, *, country: Optional[str] = None) -> schemas.AlternativeTitles:
        """Get all of the alternative titles for a movie.

        See more: https://developers.themoviedb.org/3/movies/get-movie-alternative-titles
        """
        data = self.request(f"movie/{self.movie_id}/alternative_titles", country=country)
        return utils.as_dataclass(schemas.AlternativeTitles, data)

    def credits(self) -> schemas.Credits:
        """Get the cast and crew for a movie.

        See more: https://developers.themoviedb.org/3/movies/get-movie-credits
        """
        data = self.request(f"movie/{self.movie_id}/credits")
        return utils.as_dataclass(schemas.Credits, data)

    def external_ids(self) -> schemas.ExternalIDs:
        """Get the external ids for a movie.

        See more: https://developers.themoviedb.org/3/movies/get-movie-external-ids
        """
        data = self.request(f"movie/{self.movie_id}/external_ids")
        return utils.as_dataclass(schemas.ExternalIDs, data)

    def keywords(self) -> schemas.Keywords:
        """Get the keywords that have been added to a movie.

        See more: https://developers.themoviedb.org/3/movies/get-movie-keywords
        """
        data = self.request(f"movie/{self.movie_id}/keywords")
        return utils.as_dataclass(schemas.Keywords, data)

    def images(self, *, include_image_language: Optional[str] = None) -> schemas.Images:
        """Get the images that belong to a movie.

        See more: https://developers.themoviedb.org/3/movies/get-movie-images
        """
        data = self.request(f"movie/{self.movie_id}/images", include_image_language=include_image_language)
        return utils.as_dataclass(schemas.Images, data)

    def lists(self, *, page: int = 1) -> schemas.ItemsList:
        """Get a list of lists that this movie belongs to.

        See more: https://developers.themoviedb.org/3/movies/get-movie-lists
        """
        data = self.request(f"movie/{self.movie_id}/lists", page=page)
        return utils.as_dataclass(schemas.ItemsList, data)

    def recommendations(self, *, page: int = 1) -> schemas.Movies:
        """Get a list of recommended movies for a movie.

        See more: https://developers.themoviedb.org/3/movies/get-movie-recommendations
        """
        data = self.request(f"movie/{self.movie_id}/recommendations", page=page)
        return utils.as_dataclass(schemas.Movies, data)

    def release_dates(self) -> schemas.ReleaseDates:
        """Get the release date along with the certification for a movie.

        See more: https://developers.themoviedb.org/3/movies/get-movie-release-dates
        """
        data = self.request(f"movie/{self.movie_id}/release_dates")
        return utils.as_dataclass(schemas.ReleaseDates, data)

    def reviews(self, *, page: int = 1) -> schemas.Reviews:
        """Get the user reviews for a movie.

        See more: https://developers.themoviedb.org/3/movies/get-movie-reviews
        """
        data = self.request(f"movie/{self.movie_id}/reviews", page=page)
        return utils.as_dataclass(schemas.Reviews, data)

    def similar(self, *, page: int = 1) -> schemas.Movies:
        """Get a list of similar movies.

        See more: https://developers.themoviedb.org/3/movies/get-similar-movies
        """
        data = self.request(f"movie/{self.movie_id}/similar", page=page)
        return utils.as_dataclass(schemas.Movies, data)

    def translations(self) -> schemas.Translations:
        """Get a list of translations that have been created for a movie.

        See more: https://developers.themoviedb.org/3/movies/get-movie-translations
        """
        data = self.request(f"movie/{self.movie_id}/translations")
        return utils.as_dataclass(schemas.Translations, data)

    def videos(self, *, page: int = 1) -> schemas.Videos:
        """Get the videos that have been added to a movie.

        See more: https://developers.themoviedb.org/3/movies/get-movie-videos
        """
        data = self.request(f"movie/{self.movie_id}/videos", page=page)
        return utils.as_dataclass(schemas.Videos, data)

    def watch_providers(self) -> schemas.WatchProviders:
        """Get a list of the availabilities per country by provider.

        See more: https://developers.themoviedb.org/3/movies/get-movie-watch-providers
        """
        data = self.request(f"movie/{self.movie_id}/watch/providers")
        return utils.as_dataclass(schemas.WatchProviders, data)


class Movies(Base):
    def latest(self) -> schemas.Movie:
        """Get the most newly created movie.

        See more: https://developers.themoviedb.org/3/movies/get-latest-movie
        """
        data = self.request("movie/latest")
        return utils.as_dataclass(schemas.Movie, data)

    def now_playing(self, *, page: int = 1) -> schemas.Movies:
        """Get a list of movies in theatres.

        See more: https://developers.themoviedb.org/3/movies/get-now-playing
        """
        data = self.request("movie/now_playing", page=page)
        return utils.as_dataclass(schemas.Movies, data)

    def popular(self, *, page: int = 1) -> schemas.Movies:
        """Get a list of the current popular movies on TMDB.

        See more: https://developers.themoviedb.org/3/movies/get-popular-movies
        """
        data = self.request("movie/popular", page=page)
        return utils.as_dataclass(schemas.Movies, data)

    def top_rated(self, *, page: int = 1) -> schemas.Movies:
        """Get the top rated movies on TMDB.

        See more: https://developers.themoviedb.org/3/movies/get-top-rated-movies
        """
        data = self.request("movie/top_rated", page=page)
        return utils.as_dataclass(schemas.Movies, data)

    def upcoming(self, *, page: int = 1) -> schemas.Movies:
        """Get a list of upcoming movies in theatres.

        See more: https://developers.themoviedb.org/3/movies/get-upcoming
        """
        data = self.request("movie/upcoming", page=page)
        return utils.as_dataclass(schemas.Movies, data)



# -*- coding: utf-8 -*-
from themoviedb.routes_async._base import Base  # noqa: F401
from themoviedb.routes_async.authentication import Authentication  # noqa: F401
from themoviedb.routes_async.certifications import Certifications  # noqa: F401
from themoviedb.routes_async.collections import Collection  # noqa: F401
from themoviedb.routes_async.companies import Company  # noqa: F401
from themoviedb.routes_async.credit import Credit  # noqa: F401
from themoviedb.routes_async.discover import Discover  # noqa: F401
from themoviedb.routes_async.episode_groups import EpisodeGroup  # noqa: F401
from themoviedb.routes_async.episodes import Episode  # noqa: F401
from themoviedb.routes_async.find import Find  # noqa: F401
from themoviedb.routes_async.genres import Genres  # noqa: F401
from themoviedb.routes_async.guest import Guest  # noqa: F401
from themoviedb.routes_async.keywords import Keyword  # noqa: F401
from themoviedb.routes_async.movies import Movie, Movies  # noqa: F401
from themoviedb.routes_async.network import Network  # noqa: F401
from themoviedb.routes_async.people import People, Person  # noqa: F401
from themoviedb.routes_async.reviews import Review  # noqa: F401
from themoviedb.routes_async.search import Search  # noqa: F401
from themoviedb.routes_async.seasons import Season  # noqa: F401
from themoviedb.routes_async.trending import Trending  # noqa: F401
from themoviedb.routes_async.tv import TV, TVs  # noqa: F401
from themoviedb.routes_async.watch_providers import WatchProviders  # noqa: F401



# -*- coding: utf-8 -*-
from __future__ import annotations

from dataclasses import dataclass
from typing import List, Optional

from themoviedb.schemas._partial import PartialCompany
from themoviedb.schemas._result import ResultWithPage


@dataclass
class Company(PartialCompany):
    description: Optional[str] = None
    headquarters: Optional[str] = None
    homepage: Optional[str] = None
    origin_country: Optional[str] = None
    parent_company: Optional[Company] = None


@dataclass
class Companies(ResultWithPage):
    results: Optional[List[Company]] = None



# -*- coding: utf-8 -*-
from dataclasses import asdict
from datetime import date, datetime
from enum import Enum
from typing import Any, Dict, List, Type, TypeVar

from dacite import Config, from_dict

T = TypeVar("T")
DATETIME_FORMAT = "%Y-%m-%dT%H:%M:%S.%f"

config = Config(
    cast=[Enum],
    type_hooks={
        datetime: lambda x: datetime.fromisoformat(x.rstrip("Z")) if x else None,
        date: lambda x: datetime.fromisoformat(x.rstrip("Z")).date() if x else None,
    },
)


def dict_factory(data: List[Any]) -> Dict[str, Any]:
    def convert_value(value: Any) -> Any:
        if isinstance(value, Enum):
            return value.value
        if isinstance(value, datetime):
            return f"{value.strftime(DATETIME_FORMAT)[:-3]}Z"
        if isinstance(value, date):
            return value.isoformat()
        return value

    return {k: convert_value(v) for k, v in data}


def as_dataclass(data_class: Type[T], data: Dict[str, Any]) -> T:
    return from_dict(data_class, data, config=config)


def as_dict(obj: Any) -> Dict[str, Any]:
    return asdict(obj, dict_factory=dict_factory)



# -*- coding: utf-8 -*-
from dataclasses import dataclass
from typing import List, Optional

from themoviedb.schemas._result import Result


@dataclass
class Region:
    iso_3166_1: Optional[str] = None
    english_name: Optional[str] = None
    native_name: Optional[str] = None

    def __str__(self) -> str:
        return self.native_name or ""


@dataclass
class Regions(Result):
    results: Optional[List[Region]] = None



# -*- coding: utf-8 -*-
from dataclasses import dataclass
from datetime import datetime
from typing import List, Optional

from themoviedb.schemas._enums import MediaType, SizeType
from themoviedb.schemas._result import ResultWithPage


@dataclass
class AuthorDetails:
    name: Optional[str] = None
    username: Optional[str] = None
    avatar_path: Optional[str] = None
    rating: Optional[int] = None

    def __str__(self) -> str:
        return self.name or ""

    def avatar_url(self, size: SizeType = SizeType.original) -> Optional[str]:
        return f"https://image.tmdb.org/t/p/{size.value}{self.avatar_path}" if self.avatar_path else None


@dataclass
class Review:
    id: Optional[str] = None
    author: Optional[str] = None
    author_details: Optional[AuthorDetails] = None
    content: Optional[str] = None
    created_at: Optional[datetime] = None
    updated_at: Optional[datetime] = None
    url: Optional[str] = None
    iso_639_1: Optional[str] = None
    media_id: Optional[int] = None
    media_title: Optional[str] = None
    media_type: Optional[MediaType] = None


@dataclass
class Reviews(ResultWithPage):
    results: Optional[List[Review]] = None



# -*- coding: utf-8 -*-
from typing import Optional

from themoviedb import schemas, utils
from themoviedb.routes_async._base import Base


class Discover(Base):
    async def movie(
        self,
        *,
        page: int = 1,
        sort_by: str = "popularity.desc",
        certification_country: Optional[str] = None,
        certification: Optional[str] = None,
        certification__lte: Optional[str] = None,
        certification__gte: Optional[str] = None,
        include_adult: Optional[bool] = None,
        include_video: Optional[bool] = None,
        primary_release_year: Optional[int] = None,
        primary_release_date__gte: Optional[str] = None,
        primary_release_date__lte: Optional[str] = None,
        release_date__gte: Optional[str] = None,
        release_date__lte: Optional[str] = None,
        with_release_type: Optional[int] = None,
        year: Optional[int] = None,
        vote_count__gte: Optional[int] = None,
        vote_count__lte: Optional[int] = None,
        vote_average__gte: Optional[float] = None,
        vote_average__lte: Optional[float] = None,
        with_cast: Optional[str] = None,
        with_crew: Optional[str] = None,
        with_people: Optional[str] = None,
        with_companies: Optional[str] = None,
        with_genres: Optional[str] = None,
        without_genres: Optional[str] = None,
        with_keywords: Optional[str] = None,
        without_keywords: Optional[str] = None,
        with_runtime__gte: Optional[int] = None,
        with_runtime__lte: Optional[int] = None,
        with_original_language: Optional[str] = None,
        with_watch_providers: Optional[str] = None,
        watch_region: Optional[str] = None,
        with_watch_monetization_types: Optional[str] = None,
    ) -> schemas.Movies:
        """Discover movies by different types of data.

        See more: https://developers.themoviedb.org/3/discover/movie-discover
        """
        data = await self.request(
            "discover/movie",
            page=page,
            sort_by=sort_by,
            watch_region=watch_region,
            certification_country=certification_country,
            certification=certification,
            certification__lte=certification__lte,
            certification__gte=certification__gte,
            include_adult=include_adult,
            include_video=include_video,
            primary_release_year=primary_release_year,
            primary_release_date__gte=primary_release_date__gte,
            primary_release_date__lte=primary_release_date__lte,
            release_date__gte=release_date__gte,
            release_date__lte=release_date__lte,
            with_release_type=with_release_type,
            year=year,
            vote_count__gte=vote_count__gte,
            vote_count__lte=vote_count__lte,
            vote_average__gte=vote_average__gte,
            vote_average__lte=vote_average__lte,
            with_cast=with_cast,
            with_crew=with_crew,
            with_people=with_people,
            with_companies=with_companies,
            with_genres=with_genres,
            without_genres=without_genres,
            with_keywords=with_keywords,
            without_keywords=without_keywords,
            with_runtime__gte=with_runtime__gte,
            with_runtime__lte=with_runtime__lte,
            with_original_language=with_original_language,
            with_watch_providers=with_watch_providers,
            with_watch_monetization_types=with_watch_monetization_types,
        )
        return utils.as_dataclass(schemas.Movies, data)

    async def tv(
        self,
        *,
        page: int = 1,
        sort_by: str = "popularity.desc",
        air_date__gte: Optional[str] = None,
        air_date__lte: Optional[str] = None,
        first_air_date__gte: Optional[str] = None,
        first_air_date__lte: Optional[str] = None,
        first_air_date_year: Optional[int] = None,
        timezone: Optional[str] = None,
        vote_average__gte: Optional[float] = None,
        vote_count__gte: Optional[int] = None,
        with_genres: Optional[str] = None,
        with_networks: Optional[str] = None,
        without_genres: Optional[str] = None,
        with_runtime__gte: Optional[int] = None,
        with_runtime__lte: Optional[int] = None,
        include_null_first_air_dates: Optional[bool] = None,
        with_original_language: Optional[str] = None,
        without_keywords: Optional[str] = None,
        screened_theatrically: Optional[bool] = None,
        with_companies: Optional[str] = None,
        with_keywords: Optional[str] = None,
        with_watch_providers: Optional[str] = None,
        watch_region: Optional[str] = None,
        with_watch_monetization_types: Optional[str] = None,
    ) -> schemas.TVs:
        """Discover TV shows by different types of data.

        See more: https://developers.themoviedb.org/3/discover/tv-discover
        """
        data = await self.request(
            "discover/tv",
            page=page,
            sort_by=sort_by,
            watch_region=watch_region,
            air_date__gte=air_date__gte,
            air_date__lte=air_date__lte,
            first_air_date__gte=first_air_date__gte,
            first_air_date__lte=first_air_date__lte,
            first_air_date_year=first_air_date_year,
            timezone=timezone,
            vote_average__gte=vote_average__gte,
            vote_count__gte=vote_count__gte,
            with_genres=with_genres,
            with_networks=with_networks,
            without_genres=without_genres,
            with_runtime__gte=with_runtime__gte,
            with_runtime__lte=with_runtime__lte,
            include_null_first_air_dates=include_null_first_air_dates,
            with_original_language=with_original_language,
            without_keywords=without_keywords,
            screened_theatrically=screened_theatrically,
            with_companies=with_companies,
            with_keywords=with_keywords,
            with_watch_providers=with_watch_providers,
            with_watch_monetization_types=with_watch_monetization_types,
        )
        return utils.as_dataclass(schemas.TVs, data)



# -*- coding: utf-8 -*-
from dataclasses import dataclass
from typing import List, Optional

from themoviedb.schemas._result import Result


@dataclass
class Genre:
    id: Optional[int] = None
    name: Optional[str] = None

    def __str__(self) -> str:
        return self.name or ""


@dataclass
class Genres(Result):
    genres: Optional[List[Genre]] = None

    def __post_init__(self) -> None:
        self.results = self.genres



# -*- coding: utf-8 -*-
from dataclasses import dataclass, field
from typing import Dict, List, Optional

from themoviedb.schemas._result import Result


@dataclass
class Certification:
    certification: Optional[str] = None
    meaning: Optional[str] = None
    order: Optional[int] = None


@dataclass
class Certifications(Result[Optional[Dict[str, List[Certification]]]]):
    certifications: Optional[Dict[str, List[Certification]]] = field(default=None)

    def __post_init__(self) -> None:
        self.results = self.certifications

    def __bool__(self) -> bool:
        return bool(self.results)

    def __getitem__(self, region: str) -> List[Certification]:
        if self.results is None:
            raise KeyError(f"Region {region} not found")
        return self.results[region]

    @property
    def regions(self) -> List[str]:
        if self.results is None:
            return []
        return list(self.results.keys())



# -*- coding: utf-8 -*-
from dataclasses import dataclass
from datetime import date
from typing import Generic, Optional, TypeVar

T = TypeVar("T")


@dataclass
class Dates:
    maximum: Optional[date] = None
    minimum: Optional[date] = None


@dataclass
class Result(Generic[T]):
    results: Optional[T] = None

    def __bool__(self) -> bool:
        return bool(self.results)

    def __iter__(self):
        if self.results is None:
            return iter([])
        return iter(self.results)

    def __getitem__(self, index):
        if self.results is None:
            raise IndexError("Result is empty")
        return self.results[index]

    def __len__(self) -> int:
        if self.results is None:
            return 0
        return len(self.results)  # type: ignore


@dataclass
class ResultWithID(Result):
    id: Optional[int] = None


@dataclass
class ResultWithPage(ResultWithID):
    page: Optional[int] = None
    dates: Optional[Dates] = None
    total_pages: Optional[int] = None
    total_results: Optional[int] = None



# -*- coding: utf-8 -*-
from themoviedb import schemas, utils
from themoviedb.routes_sync._base import Base


class Certifications(Base):
    def movie(self) -> schemas.Certifications:
        """Get an up to date list of the officially supported movie certifications on TMDB.

        See more: https://developers.themoviedb.org/3/certifications/get-movie-certifications
        """
        data = self.request("certification/movie/list")
        return utils.as_dataclass(schemas.Certifications, data)

    def tv(self) -> schemas.Certifications:
        """Get an up to date list of the officially supported TV show certifications on TMDB.

        See more: https://developers.themoviedb.org/3/certifications/get-tv-certifications
        """
        data = self.request("certification/tv/list")
        return utils.as_dataclass(schemas.Certifications, data)



# -*- coding: utf-8 -*-
from dataclasses import dataclass
from typing import List, Optional

from themoviedb.schemas._partial import PartialKeyword
from themoviedb.schemas._result import ResultWithPage


@dataclass
class Keyword(PartialKeyword):
    ...


@dataclass
class Keywords(ResultWithPage):
    keywords: Optional[List[Keyword]] = None
    results: Optional[List[Keyword]] = None

    def __post_init__(self) -> None:
        self.results = self.keywords or self.results



# -*- coding: utf-8 -*-
from themoviedb.routes_sync._base import Base  # noqa: F401
from themoviedb.routes_sync.authentication import Authentication  # noqa: F401
from themoviedb.routes_sync.certifications import Certifications  # noqa: F401
from themoviedb.routes_sync.collections import Collection  # noqa: F401
from themoviedb.routes_sync.companies import Company  # noqa: F401
from themoviedb.routes_sync.credit import Credit  # noqa: F401
from themoviedb.routes_sync.discover import Discover  # noqa: F401
from themoviedb.routes_sync.episode_groups import EpisodeGroup  # noqa: F401
from themoviedb.routes_sync.episodes import Episode  # noqa: F401
from themoviedb.routes_sync.find import Find  # noqa: F401
from themoviedb.routes_sync.genres import Genres  # noqa: F401
from themoviedb.routes_sync.guest import Guest  # noqa: F401
from themoviedb.routes_sync.keywords import Keyword  # noqa: F401
from themoviedb.routes_sync.movies import Movie, Movies  # noqa: F401
from themoviedb.routes_sync.network import Network  # noqa: F401
from themoviedb.routes_sync.people import People, Person  # noqa: F401
from themoviedb.routes_sync.reviews import Review  # noqa: F401
from themoviedb.routes_sync.search import Search  # noqa: F401
from themoviedb.routes_sync.seasons import Season  # noqa: F401
from themoviedb.routes_sync.trending import Trending  # noqa: F401
from themoviedb.routes_sync.tv import TV, TVs  # noqa: F401
from themoviedb.routes_sync.watch_providers import WatchProviders  # noqa: F401



# -*- coding: utf-8 -*-
from themoviedb.schemas._enums import (  # noqa: F401
    CreditType,
    EpisodeGroupType,
    ImageType,
    MediaType,
    SizeType,
)
from themoviedb.schemas._partial import (  # noqa: F401
    PartialCollection,
    PartialCompany,
    PartialKeyword,
    PartialMedia,
    PartialMovie,
    PartialPerson,
    PartialTV,
)
from themoviedb.schemas._result import (  # noqa: F401
    Dates,
    Result,
    ResultWithID,
    ResultWithPage,
)
from themoviedb.schemas.alternative_names import (  # noqa: F401
    AlternativeName,
    AlternativeNames,
)
from themoviedb.schemas.alternative_titles import (  # noqa: F401
    AlternativeTitle,
    AlternativeTitles,
)
from themoviedb.schemas.authentication import (  # noqa: F401
    Authentication,
    GuestAuthentication,
    Response,
    Session,
    TokenAuthentication,
)
from themoviedb.schemas.certifications import (  # noqa: F401
    Certification,
    Certifications,
)
from themoviedb.schemas.changes import Change  # noqa: F401
from themoviedb.schemas.collections import Collection, Collections  # noqa: F401
from themoviedb.schemas.companies import Companies, Company  # noqa: F401
from themoviedb.schemas.content_ratings import (  # noqa: F401
    ContentRating,
    ContentRatings,
)
from themoviedb.schemas.credit import Credit  # noqa: F401
from themoviedb.schemas.credits import (  # noqa: F401
    Cast,
    CastCombined,
    CastMovie,
    CastTV,
    Credits,
    CreditsCombined,
    CreditsMovie,
    CreditsTV,
    Crew,
    CrewCombined,
    CrewMovie,
    CrewTV,
)
from themoviedb.schemas.episode_groups import EpisodeGroup, EpisodeGroups  # noqa: F401
from themoviedb.schemas.episodes import Episode  # noqa: F401
from themoviedb.schemas.external_ids import ExternalIDs  # noqa: F401
from themoviedb.schemas.genres import Genre, Genres  # noqa: F401
from themoviedb.schemas.images import (  # noqa: F401
    Image,
    Images,
    TaggedImage,
    TaggedImages,
)
from themoviedb.schemas.keywords import Keyword, Keywords  # noqa: F401
from themoviedb.schemas.list import ItemList, ItemsList  # noqa: F401
from themoviedb.schemas.movies import Movie, Movies  # noqa: F401
from themoviedb.schemas.multi import Multi, MultiResults, Multis  # noqa: F401
from themoviedb.schemas.networks import Network  # noqa: F401
from themoviedb.schemas.people import People, Person  # noqa: F401
from themoviedb.schemas.rated import (  # noqa: F401
    RatedEpisode,
    RatedEpisodes,
    RatedMovie,
    RatedMovies,
    RatedTV,
    RatedTVs,
)
from themoviedb.schemas.regions import Region, Regions  # noqa: F401
from themoviedb.schemas.release_date import ReleaseDate, ReleaseDates  # noqa: F401
from themoviedb.schemas.reviews import Review, Reviews  # noqa: F401
from themoviedb.schemas.seasons import Season  # noqa: F401
from themoviedb.schemas.translations import Translation, Translations  # noqa: F401
from themoviedb.schemas.tv import TV, Episodes, TVs  # noqa: F401
from themoviedb.schemas.videos import Video, Videos  # noqa: F401
from themoviedb.schemas.watch_providers import (  # noqa: F401
    WatchProvider,
    WatchProviderData,
    WatchProviders,
    WatchProvidersData,
)



# -*- coding: utf-8 -*-
from dataclasses import dataclass
from typing import List, Optional

from themoviedb.schemas._partial import PartialEpisode, PartialMovie, PartialTV
from themoviedb.schemas._result import ResultWithPage


@dataclass
class RatedMovie(PartialMovie):
    rating: Optional[float] = None


@dataclass
class RatedMovies(ResultWithPage):
    results: Optional[List[RatedMovie]] = None


@dataclass
class RatedTV(PartialTV):
    rating: Optional[float] = None


@dataclass
class RatedTVs(ResultWithPage):
    results: Optional[List[RatedTV]] = None


@dataclass
class RatedEpisode(PartialEpisode):
    rating: Optional[float] = None


@dataclass
class RatedEpisodes(ResultWithPage):
    results: Optional[List[RatedEpisode]] = None



# -*- coding: utf-8 -*-
from typing import Optional

from themoviedb import schemas, utils
from themoviedb.routes_async._base import Base


class WatchProviders(Base):
    async def movie(self, watch_region: Optional[str] = None) -> schemas.WatchProvidersData:
        """Returns a list of the watch provider (OTT/streaming) data we have available for movies.

        See more: https://developers.themoviedb.org/3/watch-providers/get-movie-providers
        """
        data = await self.request("watch/providers/movie", watch_region=watch_region)
        return utils.as_dataclass(schemas.WatchProvidersData, data)

    async def regions(self) -> schemas.Regions:
        """Returns a list of all of the countries we have watch provider (OTT/streaming) data for.

        See more: https://developers.themoviedb.org/3/watch-providers/get-available-regions
        """
        data = await self.request("watch/providers/regions")
        return utils.as_dataclass(schemas.Regions, data)

    async def tv(self, watch_region: Optional[str] = None) -> schemas.WatchProvidersData:
        """Returns a list of the watch provider (OTT/streaming) data we have available for TV series.

        See more: https://developers.themoviedb.org/3/watch-providers/get-tv-providers
        """
        data = await self.request("watch/providers/tv", watch_region=watch_region)
        return utils.as_dataclass(schemas.WatchProvidersData, data)



# -*- coding: utf-8 -*-
from typing import Optional

from themoviedb import schemas, utils
from themoviedb.routes_async._base import Base


class Guest(Base):
    def __init__(self, guest_session_id: str, **kwargs) -> None:
        super().__init__(**kwargs)
        self.guest_session_id = guest_session_id

    async def rated_movies(self, *, sort_by: Optional[str] = None) -> schemas.RatedMovies:
        """Get the rated movies for a guest session.

        See more: https://developers.themoviedb.org/3/guest-sessions/get-guest-session-rated-movies
        """
        data = await self.request(f"guest_session/{self.guest_session_id}/rated/movies")
        return utils.as_dataclass(schemas.RatedMovies, data)

    async def rated_tvs(self, *, sort_by: Optional[str] = None) -> schemas.RatedTVs:
        """Get the rated TV shows for a guest session.

        See more: https://developers.themoviedb.org/3/guest-sessions/get-guest-session-rated-tv-shows
        """
        data = await self.request(f"guest_session/{self.guest_session_id}/rated/tv")
        return utils.as_dataclass(schemas.RatedTVs, data)

    async def rated_episodes(self, *, sort_by: Optional[str] = None) -> schemas.RatedEpisodes:
        """Get the rated TV episodes for a guest session.

        See more: https://developers.themoviedb.org/3/guest-sessions/get-gest-session-rated-tv-episodes
        """
        data = await self.request(f"guest_session/{self.guest_session_id}/rated/episodes")
        return utils.as_dataclass(schemas.RatedEpisodes, data)



# -*- coding: utf-8 -*-
from dataclasses import dataclass
from typing import Optional

from themoviedb.schemas._partial import PartialNetwork


@dataclass
class Network(PartialNetwork):
    headquarters: Optional[str] = None
    homepage: Optional[str] = None



# -*- coding: utf-8 -*-
from dataclasses import dataclass
from typing import List, Optional

from themoviedb.schemas._result import ResultWithID


@dataclass
class AlternativeTitle:
    iso_3166_1: Optional[str] = None
    title: Optional[str] = None
    type: Optional[str] = None

    def __str__(self) -> str:
        return self.title or ""


@dataclass
class AlternativeTitles(ResultWithID):
    results: Optional[List[AlternativeTitle]] = None
    titles: Optional[List[AlternativeTitle]] = None

    def __post_init__(self) -> None:
        if not self.results and self.titles:
            self.results = self.titles



# -*- coding: utf-8 -*-
from setuptools import find_packages, setup

with open("README.rst", "r", encoding="utf-8") as readme_file:
    readme = readme_file.read()

with open("requirements.txt") as f:
    requirements = f.read().splitlines()

with open("requirements-test.txt") as f:
    test_requirements = f.read().splitlines()
    test_requirements.remove("-r requirements.txt")

setup(
    name="themoviedb",
    description="A modern and easy to use API wrapper for The Movie Database (TMDb) API v3 written in Python",
    long_description=readme,
    author="Leandro César",
    author_email="ccleandroc@gmail.com",
    url="https://github.com/leandcesar/themoviedb",
    version="1.0.2",
    license="MIT",
    python_requires=">=3.8",
    packages=find_packages(include=["themoviedb", "themoviedb.*"]),
    include_package_data=True,
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
    keywords=[
        "tmdb",
        "tmdb3",
        "aiotmdb",
        "aiotmdb3",
        "themoviedb",
        "themoviedb3",
        "movie",
        "movies",
        "tv",
        "tv show",
        "tv shows",
    ],
    zip_safe=False,
    install_requires=requirements,
    tests_require=test_requirements,
    test_suite="tests",
)



# -*- coding: utf-8 -*-
from __future__ import annotations

from dataclasses import dataclass
from datetime import date, time
from typing import List, Optional

from themoviedb.schemas._enums import SizeType
from themoviedb.schemas._partial import PartialEpisode, PartialSeason, PartialTV
from themoviedb.schemas._result import ResultWithID, ResultWithPage
from themoviedb.schemas.alternative_titles import AlternativeTitles
from themoviedb.schemas.content_ratings import ContentRatings
from themoviedb.schemas.countries import Country
from themoviedb.schemas.credits import Credits
from themoviedb.schemas.episode_groups import EpisodeGroups
from themoviedb.schemas.external_ids import ExternalIDs
from themoviedb.schemas.genres import Genre
from themoviedb.schemas.images import Images
from themoviedb.schemas.keywords import Keywords
from themoviedb.schemas.languages import Language
from themoviedb.schemas.networks import Network
from themoviedb.schemas.reviews import Reviews
from themoviedb.schemas.translations import Translations
from themoviedb.schemas.videos import Videos
from themoviedb.schemas.watch_providers import WatchProviders


@dataclass
class CreatedBy:
    id: Optional[int] = None
    credit_id: Optional[str] = None
    name: Optional[str] = None
    gender: Optional[int] = None
    profile_path: Optional[str] = None

    def __str__(self) -> str:
        return self.name or ""

    def profile_url(self, size: SizeType = SizeType.original) -> Optional[str]:
        return f"https://image.tmdb.org/t/p/{size.value}{self.profile_path}" if self.profile_path else None


@dataclass
class TVs(ResultWithPage):
    results: Optional[List[PartialTV]] = None


@dataclass
class Episodes(ResultWithID):
    results: Optional[List[PartialEpisode]] = None


@dataclass
class TV(PartialTV):
    created_by: Optional[List[CreatedBy]] = None
    episode_run_time: Optional[List[int]] = None
    genres: Optional[List[Genre]] = None
    homepage: Optional[str] = None
    in_production: Optional[bool] = None
    languages: Optional[List[str]] = None
    last_air_date: Optional[date] = None
    last_episode_to_air: Optional[PartialEpisode] = None
    next_episode_to_air: Optional[PartialEpisode] = None
    networks: Optional[List[Network]] = None
    number_of_episodes: Optional[int] = None
    number_of_seasons: Optional[int] = None
    production_companies: Optional[List[Network]] = None
    production_countries: Optional[List[Country]] = None
    seasons: Optional[List[PartialSeason]] = None
    spoken_languages: Optional[List[Language]] = None
    status: Optional[str] = None
    tagline: Optional[str] = None
    type: Optional[str] = None
    aggregate_credits: Optional[Credits] = None
    alternative_titles: Optional[AlternativeTitles] = None
    # changes: Optional[] = None  # TODO
    content_ratings: Optional[ContentRatings] = None
    credits: Optional[Credits] = None
    external_ids: Optional[ExternalIDs] = None
    episode_groups: Optional[EpisodeGroups] = None
    images: Optional[Images] = None
    keywords: Optional[Keywords] = None
    recommendations: Optional[TVs] = None
    reviews: Optional[Reviews] = None
    screened_theatrically: Optional[Episodes] = None
    similar: Optional[TVs] = None
    translations: Optional[Translations] = None
    videos: Optional[Videos] = None
    watch_providers: Optional[WatchProviders] = None

    @property
    def run_time(self) -> Optional[int]:
        if self.episode_run_time and self.number_of_episodes:
            run_time = self.episode_run_time[0] * self.number_of_episodes
            return run_time
        return None

    def episode_duration(self, fmt: str = "%H:%M") -> Optional[str]:
        if self.run_time and self.number_of_episodes:
            run_time_average = self.run_time // self.number_of_episodes
            return time(minute=run_time_average).strftime(fmt)
        return None

    def duration(self, fmt: str = "%H:%M") -> Optional[str]:
        if self.run_time:
            return time(minute=self.run_time).strftime(fmt)
        return None



# -*- coding: utf-8 -*-
from themoviedb import schemas, utils
from themoviedb.routes_async._base import Base


class Review(Base):
    def __init__(self, review_id: int, **kwargs) -> None:
        super().__init__(**kwargs)
        self.review_id = review_id

    async def details(self) -> schemas.Review:
        """Retrieve the details of a movie or TV show review.

        See more: https://developers.themoviedb.org/3/reviews/get-review-details
        """
        data = await self.request(f"review/{self.review_id}")
        return utils.as_dataclass(schemas.Review, data)



# -*- coding: utf-8 -*-
from dataclasses import dataclass
from datetime import date
from typing import List, Optional

from themoviedb.schemas._partial import PartialPerson
from themoviedb.schemas._result import ResultWithPage
from themoviedb.schemas.credits import CreditsCombined, CreditsMovie, CreditsTV
from themoviedb.schemas.external_ids import ExternalIDs
from themoviedb.schemas.images import Images, TaggedImages
from themoviedb.schemas.translations import Translations


@dataclass
class Person(PartialPerson):
    birthday: Optional[date] = None
    deathday: Optional[date] = None
    also_known_as: Optional[List[str]] = None
    biography: Optional[str] = None
    place_of_birth: Optional[str] = None
    imdb_id: Optional[str] = None
    homepage: Optional[str] = None
    external_ids: Optional[ExternalIDs] = None
    images: Optional[Images] = None
    combined_credits: Optional[CreditsCombined] = None
    movie_credits: Optional[CreditsMovie] = None
    tv_credits: Optional[CreditsTV] = None
    tagged_images: Optional[TaggedImages] = None
    translations: Optional[Translations] = None

    @property
    def imdb_url(self) -> Optional[str]:
        return f"https://www.imdb.com/name/{self.imdb_id}" if self.imdb_id else None


@dataclass
class People(ResultWithPage):
    results: Optional[List[Person]] = None



# -*- coding: utf-8 -*-
from themoviedb import schemas, utils
from themoviedb.routes_async._base import Base


class Trending(Base):
    async def movie_daily(self, *, page: int = 1) -> schemas.Movies:
        """Get the daily trending movies.

        See more: https://developers.themoviedb.org/3/trending/get-trending
        """
        data = await self.request("trending/movie/day", page=page)
        return utils.as_dataclass(schemas.Movies, data)

    async def movie_weekly(self, *, page: int = 1) -> schemas.Movies:
        """Get the weekly trending movies.

        See more: https://developers.themoviedb.org/3/trending/get-trending
        """
        data = await self.request("trending/movie/week", page=page)
        return utils.as_dataclass(schemas.Movies, data)

    async def person_daily(self, *, page: int = 1) -> schemas.People:
        """Get the daily trending people.

        See more: https://developers.themoviedb.org/3/trending/get-trending
        """
        data = await self.request("trending/person/day", page=page)
        return utils.as_dataclass(schemas.People, data)

    async def person_weekly(self, *, page: int = 1) -> schemas.People:
        """Get the weekly trending people.

        See more: https://developers.themoviedb.org/3/trending/get-trending
        """
        data = await self.request("trending/person/week", page=page)
        return utils.as_dataclass(schemas.People, data)

    async def tv_daily(self, *, page: int = 1) -> schemas.TVs:
        """Get the daily trending TV shows.

        See more: https://developers.themoviedb.org/3/trending/get-trending
        """
        data = await self.request("trending/tv/day", page=page)
        return utils.as_dataclass(schemas.TVs, data)

    async def tv_weekly(self, *, page: int = 1) -> schemas.TVs:
        """Get the weekly trending TV shows.

        See more: https://developers.themoviedb.org/3/trending/get-trending
        """
        data = await self.request("trending/tv/week", page=page)
        return utils.as_dataclass(schemas.TVs, data)



# -*- coding: utf-8 -*-
from dataclasses import dataclass
from typing import Optional, Union

from themoviedb.schemas._enums import CreditType, MediaType
from themoviedb.schemas._partial import PartialPerson
from themoviedb.schemas.credits import CastMovie, CastTV
from themoviedb.schemas.movies import Movie
from themoviedb.schemas.seasons import Season
from themoviedb.schemas.tv import TV


@dataclass
class FullTV(CastTV, TV, Season):
    ...


@dataclass
class FullMovie(CastMovie, Movie):
    ...


@dataclass
class Credit:
    id: Optional[str] = None
    credit_type: Optional[CreditType] = None
    department: Optional[str] = None
    job: Optional[str] = None
    media: Optional[Union[FullTV, FullMovie]] = None
    media_type: Optional[MediaType] = None
    person: Optional[PartialPerson] = None
